body,
html {
    margin: 0;
    padding: 0;
}

#stpyvistadiv {
    text-align: center;
    display: block;
    margin-left: 0px;
    margin-right: 0px;
    width: 100%;
    background-color: rgba(255, 0, 0, 0);
}

#toggle-fullscreen {
    background-color: #ffffff4e;
    border: #ffffff00 solid 0px;
    border-radius: 10%;
    font-size: 24px;
    position: absolute;
    top: 10px;
    right: calc(100% - (100% - 10px));
    padding-top: 0;
    padding-bottom: 0;
    opacity: 0.5;
    transition: 0.3s;
}

#toggle-fullscreen:hover {
    box-shadow: 0 12px 16px 0 rgba(0, 0, 0, 0.24), 0 17px 50px 0 rgba(0, 0, 0, 0.19);
    opacity: 0.8;
}